package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.task.domain.JOBRegistry;
import cn.ibizlab.core.task.service.IJOBRegistryService;
import cn.ibizlab.core.task.filter.JOBRegistrySearchContext;

@Slf4j
@Api(tags = {"任务注册信息" })
@RestController("api-jobregistry")
@RequestMapping("")
public class JOBRegistryResource {

    @Autowired
    public IJOBRegistryService jobregistryService;

    @Autowired
    @Lazy
    public JOBRegistryMapping jobregistryMapping;

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Get-all')")
    @ApiOperation(value = "获取任务注册信息", tags = {"任务注册信息" },  notes = "获取任务注册信息")
	@RequestMapping(method = RequestMethod.GET, value = "/jobregistries/{jobregistry_id}")
    public ResponseEntity<JOBRegistryDTO> get(@PathVariable("jobregistry_id") BigInteger jobregistry_id) {
        JOBRegistry domain = jobregistryService.get(jobregistry_id);
        JOBRegistryDTO dto = jobregistryMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Remove-all')")
    @ApiOperation(value = "删除任务注册信息", tags = {"任务注册信息" },  notes = "删除任务注册信息")
	@RequestMapping(method = RequestMethod.DELETE, value = "/jobregistries/{jobregistry_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("jobregistry_id") BigInteger jobregistry_id) {
         return ResponseEntity.status(HttpStatus.OK).body(jobregistryService.remove(jobregistry_id));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Remove-all')")
    @ApiOperation(value = "批量删除任务注册信息", tags = {"任务注册信息" },  notes = "批量删除任务注册信息")
	@RequestMapping(method = RequestMethod.DELETE, value = "/jobregistries/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<BigInteger> ids) {
        jobregistryService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Update-all')")
    @ApiOperation(value = "更新任务注册信息", tags = {"任务注册信息" },  notes = "更新任务注册信息")
	@RequestMapping(method = RequestMethod.PUT, value = "/jobregistries/{jobregistry_id}")
    @Transactional
    public ResponseEntity<JOBRegistryDTO> update(@PathVariable("jobregistry_id") BigInteger jobregistry_id, @RequestBody JOBRegistryDTO jobregistrydto) {
		JOBRegistry domain  = jobregistryMapping.toDomain(jobregistrydto);
        domain .setId(jobregistry_id);
		jobregistryService.update(domain );
		JOBRegistryDTO dto = jobregistryMapping.toDto(domain );
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Update-all')")
    @ApiOperation(value = "批量更新任务注册信息", tags = {"任务注册信息" },  notes = "批量更新任务注册信息")
	@RequestMapping(method = RequestMethod.PUT, value = "/jobregistries/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<JOBRegistryDTO> jobregistrydtos) {
        jobregistryService.updateBatch(jobregistryMapping.toDomain(jobregistrydtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Save-all')")
    @ApiOperation(value = "保存任务注册信息", tags = {"任务注册信息" },  notes = "保存任务注册信息")
	@RequestMapping(method = RequestMethod.POST, value = "/jobregistries/save")
    public ResponseEntity<Boolean> save(@RequestBody JOBRegistryDTO jobregistrydto) {
        return ResponseEntity.status(HttpStatus.OK).body(jobregistryService.save(jobregistryMapping.toDomain(jobregistrydto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Save-all')")
    @ApiOperation(value = "批量保存任务注册信息", tags = {"任务注册信息" },  notes = "批量保存任务注册信息")
	@RequestMapping(method = RequestMethod.POST, value = "/jobregistries/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<JOBRegistryDTO> jobregistrydtos) {
        jobregistryService.saveBatch(jobregistryMapping.toDomain(jobregistrydtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Create-all')")
    @ApiOperation(value = "新建任务注册信息", tags = {"任务注册信息" },  notes = "新建任务注册信息")
	@RequestMapping(method = RequestMethod.POST, value = "/jobregistries")
    @Transactional
    public ResponseEntity<JOBRegistryDTO> create(@RequestBody JOBRegistryDTO jobregistrydto) {
        JOBRegistry domain = jobregistryMapping.toDomain(jobregistrydto);
		jobregistryService.create(domain);
        JOBRegistryDTO dto = jobregistryMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Create-all')")
    @ApiOperation(value = "批量新建任务注册信息", tags = {"任务注册信息" },  notes = "批量新建任务注册信息")
	@RequestMapping(method = RequestMethod.POST, value = "/jobregistries/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<JOBRegistryDTO> jobregistrydtos) {
        jobregistryService.createBatch(jobregistryMapping.toDomain(jobregistrydtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "检查任务注册信息", tags = {"任务注册信息" },  notes = "检查任务注册信息")
	@RequestMapping(method = RequestMethod.POST, value = "/jobregistries/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody JOBRegistryDTO jobregistrydto) {
        return  ResponseEntity.status(HttpStatus.OK).body(jobregistryService.checkKey(jobregistryMapping.toDomain(jobregistrydto)));
    }

    @ApiOperation(value = "获取任务注册信息草稿", tags = {"任务注册信息" },  notes = "获取任务注册信息草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/jobregistries/getdraft")
    public ResponseEntity<JOBRegistryDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(jobregistryMapping.toDto(jobregistryService.getDraft(new JOBRegistry())));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Default-all')")
	@ApiOperation(value = "获取DEFAULT", tags = {"任务注册信息" } ,notes = "获取DEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/jobregistries/fetchdefault")
	public ResponseEntity<List<JOBRegistryDTO>> fetchDefault(JOBRegistrySearchContext context) {
        Page<JOBRegistry> domains = jobregistryService.searchDefault(context) ;
        List<JOBRegistryDTO> list = jobregistryMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBRegistry-Default-all')")
	@ApiOperation(value = "查询DEFAULT", tags = {"任务注册信息" } ,notes = "查询DEFAULT")
    @RequestMapping(method= RequestMethod.POST , value="/jobregistries/searchdefault")
	public ResponseEntity<Page<JOBRegistryDTO>> searchDefault(@RequestBody JOBRegistrySearchContext context) {
        Page<JOBRegistry> domains = jobregistryService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(jobregistryMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}

